package com.gx.obe.common.composite;

import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CLabel;
import org.eclipse.swt.events.FocusAdapter;
import org.eclipse.swt.events.FocusEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;

import com.gx.obe.ColorConstants;
import com.gx.obe.common.icons.IconsActivator;
import com.gx.obe.components.core.enumeration.BidPriceEnum;
import com.gx.obe.components.core.util.MoneyUtils;
import com.gx.obe.util.utils.NumberFormatUtils;
import com.gx.obe.util.utils.ObjectUtils;
import com.gx.obe.web.entity.price.BidPriceResult;
import com.swtdesigner.ResourceManager;
import com.swtdesigner.SWTResourceManager;

public class BidPriceResultComposite extends Composite {

	private CLabel bidPriceResultTitle_label;
	private Label bidPriceResultUnit_label;
	private Text bidPriceResult_text;
	private BidPriceResult bidPriceResult;
	/**
	 * Create the composite.
	 * @param parent
	 * @param style
	 */
	public BidPriceResultComposite(Composite parent, int style) {
		super(parent, style);
		setLayout(new GridLayout(3, false));
		
		bidPriceResultTitle_label = new CLabel(this, SWT.WRAP);
		bidPriceResultTitle_label.setAlignment(SWT.RIGHT);
		bidPriceResultTitle_label.setFont(SWTResourceManager.getFont("微软雅黑", 10, SWT.BOLD));
		bidPriceResultTitle_label.setForeground(ColorConstants.NOTE_TIPS_COLOR);
		
		GridData gd_title = new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1);
		gd_title.widthHint = 100;
		bidPriceResultTitle_label.setLayoutData(gd_title);
		
		bidPriceResult_text = new Text(this, SWT.BORDER);
		GridData gd_value = new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1);
		gd_value.widthHint = 100;
		
		bidPriceResult_text.setLayoutData(gd_value);
		bidPriceResult_text.setFont(SWTResourceManager.getFont("微软雅黑", 10, SWT.NORMAL));
		
		bidPriceResult_text.setForeground(ColorConstants.FONT_BLACK_COLOR);
		bidPriceResult_text.addFocusListener(new FocusAdapter(){
			public void focusGained(FocusEvent e) {
				bidPriceResult_text.setForeground(ColorConstants.SKIN_BG);
			}
			public void focusLost(FocusEvent e) {
				bidPriceResult_text.setForeground(ColorConstants.FONT_BLACK_COLOR);
			}
		});
		
		bidPriceResultUnit_label = new Label(this, SWT.WRAP);
		bidPriceResultUnit_label.setFont(SWTResourceManager.getFont("微软雅黑", 10, SWT.BOLD));
		GridData gd_unit = new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1);
		gd_unit.widthHint = 30;
		bidPriceResultUnit_label.setLayoutData(gd_unit);
		bidPriceResultUnit_label.setForeground(ColorConstants.NOTE_TIPS_COLOR);
		
	}

	public void refreshBidPriceResult(BidPriceResult result, String defaultUnit){
		this.bidPriceResult = result;
		bidPriceResultTitle_label.setText(ObjectUtils.getObjString(bidPriceResult.getBidPriceName())+ "：");
		if(BidPriceEnum.IS_PRICE_ITEM.equals(bidPriceResult.getBidPriceType())){
			if(BidPriceEnum.IS_RATE.equals(bidPriceResult.getPriceType())){
				String priceUnit = "%";//StringUtils.hasLength(bidPriceResult.getPriceUnit()) ? bidPriceResult.getPriceUnit() : "%";
				if(null != bidPriceResult.getBidPrice()) {
					bidPriceResult_text.setText(NumberFormatUtils.format(bidPriceResult.getBidPrice()));
				}
				bidPriceResultUnit_label.setText(ObjectUtils.getObjString(priceUnit));
			}else {
				bidPriceResultTitle_label.setImage(ResourceManager.getPluginImage(IconsActivator.PLUGIN_ID, "icons/money-yen.png"));
				bidPriceResult_text.setText(bidPriceResult.getFormatBidPriceByUnit(defaultUnit));
				bidPriceResultUnit_label.setText(ObjectUtils.getObjString(bidPriceResult.getPriceUnit(defaultUnit)));
			}
		}else {
			bidPriceResult_text.setText(ObjectUtils.getObjString(bidPriceResult.getPriceContent()));
		}
	}
	
	
	public void refreshBidPriceUnit(String currencyCode, String currencyName, String priceUnit){
		if(BidPriceEnum.IS_PRICE_ITEM.equals(bidPriceResult.getBidPriceType())){
			if((!"%".equals(priceUnit) && BidPriceEnum.IS_RATE.equals(bidPriceResult.getPriceType())) || ("%".equals(priceUnit) && BidPriceEnum.IS_PRICE.equals(bidPriceResult.getPriceType()))) {
				return;
			}
			if(BidPriceEnum.IS_RATE.equals(bidPriceResult.getPriceType())){
				bidPriceResult.setCurrencyCode(null);
				bidPriceResult.setCurrencyName(null);
				bidPriceResult.setPriceUnit("%");
				bidPriceResultUnit_label.setText("%");
				bidPriceResult_text.setText(NumberFormatUtils.format(bidPriceResult.getBidPrice()));
			}else {
				bidPriceResult.setCurrencyCode(currencyCode);
				bidPriceResult.setCurrencyName(currencyName);
				bidPriceResult.setPriceUnit(priceUnit);
				bidPriceResultUnit_label.setText(ObjectUtils.getObjString(bidPriceResult.getPriceUnit(priceUnit)));
				bidPriceResult_text.setText(bidPriceResult.getFormatBidPriceByUnit(priceUnit));
			}
		}
	}
	
	public BidPriceResult getBidPriceResult(){
		if(BidPriceEnum.IS_PRICE_ITEM.equals(bidPriceResult.getBidPriceType())){
			if(BidPriceEnum.IS_RATE.equals(bidPriceResult.getPriceType())) {
				bidPriceResult.setBidPrice(ObjectUtils.getObjBigDecimal(bidPriceResult_text.getText().trim(), 6));
			}else {
				bidPriceResult.setBidPrice(MoneyUtils.getMoneyFromText(bidPriceResult_text.getText().trim(), bidPriceResult.getPriceUnit()));
			}
		}else {
			bidPriceResult.setPriceContent(bidPriceResult_text.getText().trim());
		}
		return bidPriceResult;
	}
	@Override
	protected void checkSubclass() {
		// Disable the check that prevents subclassing of SWT components
	}

}
