package com.gx.obe.common.composite;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;

import com.gx.obe.ColorConstants;
import com.gx.obe.components.core.I18NEnum;
import com.gx.obe.web.entity.EvaluationFactorResult;
import com.gx.obe.business.entity.SupplierEvaluationStep;
import com.gx.obe.components.core.enumeration.EvaluationFactorEnum;
import com.gx.obe.components.core.Constants;
import com.gx.obe.config.utils.PropertiesUtils;
import com.gx.obe.util.utils.BigDecimalUtils;
import com.gx.obe.util.utils.LogUtils;
import com.gx.obe.util.utils.ObjectUtils;
import com.gx.obe.web.entity.Supplier;
import com.gx.obe.web.entity.evaluation.EvaluationFactor;
import com.gx.obe.web.entity.evaluation.EvaluationStep;
import com.gx.obe.web.entity.tender.TenderProjectEntity;
import com.gx.obe.web.service.SupplierEvaluationStepService;
import com.gx.obe.web.service.evaluation.EvaluationFactorResultService;
import com.gx.obe.web.service.evaluation.EvaluationFactorService;
import com.swtdesigner.SWTResourceManager;

import de.kupzog.ktable.KTable;
import de.kupzog.ktable.SWTX;
import de.kupzog.ktable.model.CommonKtableCModel;
import de.kupzog.ktable.renderers.CellAlignmentRender;
import de.kupzog.ktable.renderers.CellBackgroundRender;
import de.kupzog.ktable.renderers.CellContentRender;
import de.kupzog.ktable.renderers.CellForegroundRender;
import de.kupzog.ktable.renderers.CellRender;
import de.kupzog.ktable.renderers.CellTooltipRender;
import de.kupzog.ktable.renderers.FixedWrapCellRenderer;
import de.kupzog.ktable.renderers.RowHeightRender;
import de.kupzog.ktable.renderers.TextWrapCellRenderer;

/**
 * @Description: 
 * @author guoyr
 */
public class BillScoreResultComposite extends Composite {

	private SupplierEvaluationStepService supplierEvaluationStepService ;
	private EvaluationFactorService evaluationFactorService = new EvaluationFactorService();
	private EvaluationFactorResultService evaluationFactorResultService = new EvaluationFactorResultService();
	private TenderProjectEntity tenderProjectEntity;
	private List<Supplier> supplierList;
	private String supplierName;
	private Composite load_composite;
	private Label message_label;
	private Composite message_composite;
	private KTable kTable;
	private EvaluationStep evaluationStep;
	private Map<String, SupplierEvaluationStep> supplierEvaluationStepMap;
	private Map<String, EvaluationFactorResult> totalPriceFactorResultMap;
	private Map<String, EvaluationFactorResult> itemizeFactorResultMap;
	private Map<String, EvaluationFactorResult> measureFactorResultMap;
	{
		LogUtils.logClass(BillScoreResultComposite.class);
	}
	
	/**
	 * Create the composite.
	 * @param parent
	 * @param style
	 */
	public BillScoreResultComposite(Composite parent, int style) {
		super(parent, style);
		GridLayout gridLayout = new GridLayout(1, false);
		gridLayout.verticalSpacing = 0;
		gridLayout.horizontalSpacing = 0;
		gridLayout.marginHeight = 1;
		gridLayout.marginWidth = 1;
		setLayout(gridLayout);
		
		message_composite = new Composite(this, SWT.NONE);
		GridLayout gl_composite_6 = new GridLayout(2, false);
		message_composite.setLayout(gl_composite_6);
		message_composite.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
		
		message_label = new Label(message_composite, SWT.NONE);
		message_label.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false, 1, 1));
		message_label.setFont(SWTResourceManager.getFont("微软雅黑", 10, SWT.NORMAL));
		message_label.setForeground(ColorConstants.NOTE_TIPS_COLOR);
		
		Composite composite_2 = new Composite(this, SWT.NONE);
		composite_2.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
		composite_2.setBackground(ColorConstants.BORDER_LINE_COLOR);
		GridLayout gl_composite_2 = new GridLayout(1, false);
		gl_composite_2.marginHeight = 1;
		gl_composite_2.marginWidth = 1;
		composite_2.setLayout(gl_composite_2);
		
		load_composite = new Composite(composite_2, SWT.NONE);
		load_composite.setBackgroundMode(SWT.INHERIT_FORCE);
		load_composite.setBackground(SWTResourceManager.getColor(SWT.COLOR_WHITE));
		GridLayout gl_load_composite = new GridLayout(1, false);
		gl_load_composite.verticalSpacing = 0;
		gl_load_composite.horizontalSpacing = 0;
		gl_load_composite.marginHeight = 0;
		gl_load_composite.marginWidth = 0;
		load_composite.setLayout(gl_load_composite);
		load_composite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
		
		init();
	}
	
	/**
	 * @Description: 初始化
	 * @author guoyr
	 */
	private void init(){
		supplierEvaluationStepService = new SupplierEvaluationStepService();
		((GridData)message_composite.getLayoutData()).exclude = true;
		message_composite.setVisible(false);
	}
	
	public void refresh(TenderProjectEntity tenderProjectEntity, EvaluationStep evaluationStep, List<Supplier> supplierList) {
		this.tenderProjectEntity = tenderProjectEntity;
		this.evaluationStep = evaluationStep;
		this.supplierList = supplierList;
		drawEvaluationTable();
	}
	
	/**
	 * @Description: 初始化评标结果
	 * @author guoyr
	 * @param userId 
	 */
	private void initEvaluationResult(){
		supplierName = com.gx.obe.components.core.I18N.getString(I18NEnum.SUPPLIERNAME, tenderProjectEntity.getTenderType());
		
		supplierEvaluationStepMap = new HashMap<String, SupplierEvaluationStep>();
		
		for (Supplier supplier : supplierList) {
			SupplierEvaluationStep supplierEvaluationStep = supplierEvaluationStepService.getSupplierEvaluationStep(tenderProjectEntity.getId(), supplier.getId(), evaluationStep.getId());
			supplierEvaluationStepMap.put(supplier.getId(), supplierEvaluationStep);
		}
		
		EvaluationFactor totalPriceFactor = evaluationFactorService.getBillFactoryByType(tenderProjectEntity.getId(), EvaluationFactorEnum.FACTOR_TOTAL_PRICE);
		EvaluationFactor itemizeFactor = evaluationFactorService.getBillFactoryByType(tenderProjectEntity.getId(), EvaluationFactorEnum.FACTOR_ITEMIZE);
		EvaluationFactor measureFactor = evaluationFactorService.getBillFactoryByType(tenderProjectEntity.getId(), EvaluationFactorEnum.FACTOR_MEASURE);
		
		List<EvaluationFactorResult> totalPriceFactorResultList = null;
		if(null != totalPriceFactor){
			totalPriceFactorResultList = evaluationFactorResultService.getEvaluationFactorResultList(tenderProjectEntity.getId(), evaluationStep.getId(), totalPriceFactor.getId());
		}
		List<EvaluationFactorResult> itemizeFactorResultList = null;
		if(null != itemizeFactor){
			itemizeFactorResultList = evaluationFactorResultService.getEvaluationFactorResultList(tenderProjectEntity.getId(), evaluationStep.getId(), itemizeFactor.getId());
		}
		List<EvaluationFactorResult> measureFactorResultList = null;
		
		if(null != measureFactor){
			measureFactorResultList = evaluationFactorResultService.getEvaluationFactorResultList(tenderProjectEntity.getId(), evaluationStep.getId(), measureFactor.getId());
		}
		
		totalPriceFactorResultMap = getEvaluationFactorResultMap(totalPriceFactorResultList);
		itemizeFactorResultMap = getEvaluationFactorResultMap(itemizeFactorResultList);
		measureFactorResultMap = getEvaluationFactorResultMap(measureFactorResultList);
	}
	
	private Map<String, EvaluationFactorResult> getEvaluationFactorResultMap(List<EvaluationFactorResult> evaluationFactorResultMap) {
		Map<String, EvaluationFactorResult> map = new HashMap<String, EvaluationFactorResult>();
		if(null != evaluationFactorResultMap){
			for (EvaluationFactorResult evaluationFactorResult : evaluationFactorResultMap) {
				map.put(evaluationFactorResult.getSupplierId(), evaluationFactorResult);
			}
		}
		return map;
	}

	private void drawEvaluationTable(){
		for(Control control : load_composite.getChildren()){
			control.dispose();
		}
		initEvaluationResult();
		drawRankEvaluationTable();
		load_composite.layout();
	}
	
	private void drawRankEvaluationTable() {

		if(null != kTable && !kTable.isDisposed()){
			kTable.dispose();
		}
		
		final Map<String, String> supplierNameMap = new HashMap<String, String>();
		for (Supplier supplier : supplierList) {
			supplierNameMap.put(supplier.getId(), supplier.getSupplierName());
		}
		
		kTable = new KTable(load_composite, SWTX.AUTO_SCROLL | SWT.FULL_SELECTION | SWTX.FILL_WITH_LASTCOL | SWTX.EDIT_ON_KEY);
		kTable.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
		final CommonKtableCModel commonKtableCModel = new CommonKtableCModel(kTable, 1, 0, supplierList.size(), 5);
		
		//设置行高
		commonKtableCModel.setRowHeightRender(new RowHeightRender() {
			public int receiveRowHeight(int row) {
				return Constants.ROW_HIGHT;
			}
		});
		
		//设置单元格内容
		commonKtableCModel.setCellContentRender(new CellContentRender() {
			public Object receiveContent(int row, int col) {
				if (row == 0) {
					switch (col) {
					case 0: return supplierName;
					case 1: return PropertiesUtils.getEnum(I18NEnum.BILL_FACTOR_TOTAL_PRICE, "总报价") + "得分";
					case 2: return PropertiesUtils.getEnum(I18NEnum.BILL_FACTOR_TOTAL_FBFX, "分部分项") + "得分";
					case 3: return PropertiesUtils.getEnum(I18NEnum.BILL_FACTOR_TOTAL_CSXM, "措施项") + "得分";
					case 4: return "总得分";
					}
				} else {
					Supplier supplier = supplierList.get(row - 1);
					if (col == 0) {
						return supplierNameMap.get(supplier.getId());
					} else if (col == 1 && null != totalPriceFactorResultMap.get(supplier.getId())) {
						return ObjectUtils.getObjString(BigDecimalUtils.round(totalPriceFactorResultMap.get(supplier.getId()).getEvaluationScore(), 2));
					} else if (col == 2 && null != itemizeFactorResultMap.get(supplier.getId())) {
						return ObjectUtils.getObjString(BigDecimalUtils.round(itemizeFactorResultMap.get(supplier.getId()).getEvaluationScore(), 2));
					} else if (col == 3 && null != measureFactorResultMap.get(supplier.getId())) {
						return ObjectUtils.getObjString(BigDecimalUtils.round(measureFactorResultMap.get(supplier.getId()).getEvaluationScore(), 2));
					} else if (col == 4 && null != supplierEvaluationStepMap.get(supplier.getId())) {
						return ObjectUtils.getObjString(BigDecimalUtils.round(supplierEvaluationStepMap.get(supplier.getId()).getEvaluationScore(), 2));
					}
				}
				return "";
			}
		});

		final FixedWrapCellRenderer fixedWrapCellRenderer = new FixedWrapCellRenderer();
		final TextWrapCellRenderer textWrapCellRenderer = new TextWrapCellRenderer();
		
		// 设置单元格渲染器
		commonKtableCModel.setCellRender(new CellRender() {
			public Object receiveCellRender(int row, int col) {
				if (commonKtableCModel.isFixedCell(col, row)) {
					return fixedWrapCellRenderer;
				} else {
					return textWrapCellRenderer;
				}
			}
		});
		
		// 编辑单元格对齐方式
		commonKtableCModel.setCellAlignmentRender(new CellAlignmentRender() {
			public int receiveCellAlignment(int row, int col) {
				if (row == 0) {
					return SWTX.ALIGN_VERTICAL_CENTER | SWTX.ALIGN_HORIZONTAL_CENTER;
				} else {
					if (col == 0) {
						return SWTX.ALIGN_VERTICAL_CENTER | SWTX.ALIGN_HORIZONTAL_LEFT;
					} else {
						return SWTX.ALIGN_VERTICAL_CENTER | SWTX.ALIGN_HORIZONTAL_RIGHT;
					}
				}
			}
		});

		// 设置单元格前景颜色
		commonKtableCModel.setCellForegroundRender(new CellForegroundRender() {
			public Color receiveCellForeground(int row, int col) {
				return null;
			}
		});

		// 设置单元格背景颜色
		commonKtableCModel.setCellBackgroundRender(new CellBackgroundRender() {
			public Color receiveCellBackground(int row, int col) {
				return SWTResourceManager.getColor(SWT.COLOR_WHITE);
			}
		});
		
		// 设置单元格提示内容
		commonKtableCModel.setCellTooltipRender(new CellTooltipRender() {
			public String receiveCellTooltip(int row, int col) {
				return "";
			}
		});
		
		kTable.setModel(commonKtableCModel);
		load_composite.layout();
		
	}

	@Override
	protected void checkSubclass() {
		// Disable the check that prevents subclassing of SWT components
	}

}
