package com.gx.obe.common.composite;

import java.text.MessageFormat;
import java.util.List;

import org.eclipse.nebula.widgets.grid.Grid;
import org.eclipse.nebula.widgets.grid.GridColumn;
import org.eclipse.nebula.widgets.grid.GridItem;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;


import com.gx.obe.components.core.I18NEnum;
import com.gx.obe.action.bean.Action;
import com.gx.obe.message.Messages;
import com.gx.obe.web.entity.auth.Menu;
import com.gx.obe.web.entity.tender.TenderProjectEntity;
import com.gx.obe.web.service.TenderProjectService;
import com.swtdesigner.SWTResourceManager;

/**
 * @Description: 按项目下载解密投标文件
 * @author chenxw
 */
public class DecryptBidFileDownloadComposite extends Composite {
	private TenderProjectEntity tenderProjectEntity;
	private Composite packListTableComposite;
	private Composite contentComposite;
	private Grid tenderPackGrid;
	private String subPackAge;
	private TenderProjectService tenderProjectService;
	private DecryptBidFileDownloadBySupplierComposite bidFileDownloadBySupplierComposite;
	private Menu menu;
	/**
	 * Create the composite.
	 * 
	 * @param parent
	 * @param style
	 */
	public DecryptBidFileDownloadComposite(Composite parent, Action action, int style) {
		super(parent, style);
		GridLayout gridLayout = new GridLayout(1, false);
		gridLayout.verticalSpacing = 0;
		gridLayout.horizontalSpacing = 0;
		gridLayout.marginHeight = 0;
		gridLayout.marginWidth = 0;
		setLayout(gridLayout);

		tenderProjectEntity = action.getParentProject();
		menu = action.getMenu();
		subPackAge = com.gx.obe.components.core.I18N.getString(I18NEnum.SUBPACKAGE, tenderProjectEntity.getTenderType());

		contentComposite = new Composite(this, SWT.NONE);
		contentComposite.setBackground(SWTResourceManager.getColor(SWT.COLOR_WHITE));
		GridLayout gl_contentComposite = new GridLayout(2, false);
		gl_contentComposite.marginHeight = 0;
		gl_contentComposite.marginWidth = 0;
		contentComposite.setLayout(gl_contentComposite);
		contentComposite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
		contentComposite.setData("menu",menu);
		
		Composite packListComposite = new Composite(contentComposite, SWT.NONE);
		packListComposite.setBackground(SWTResourceManager.getColor(SWT.COLOR_GRAY));
		GridLayout compositeLayout = new GridLayout(1, false);
		compositeLayout.verticalSpacing = 0;
		compositeLayout.horizontalSpacing = 0;
		compositeLayout.marginHeight = 1;
		compositeLayout.marginWidth = 1;
		packListComposite.setLayout(compositeLayout);
		GridData compositeData = new GridData(SWT.FILL, SWT.FILL, false, true, 1, 1);
		compositeData.widthHint = 300;
		packListComposite.setLayoutData(compositeData);

		packListTableComposite = new Composite(packListComposite, SWT.NONE);
		packListTableComposite.setBackground(SWTResourceManager.getColor(SWT.COLOR_WHITE));
		GridLayout gl_packListTablecomposite = new GridLayout(1, false);
		gl_packListTablecomposite.verticalSpacing = 0;
		gl_packListTablecomposite.horizontalSpacing = 0;
		gl_packListTablecomposite.marginHeight = 0;
		gl_packListTablecomposite.marginWidth = 0;
		packListTableComposite.setLayout(gl_packListTablecomposite);
		packListTableComposite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));

		tenderPackGrid = new Grid(packListTableComposite, SWT.V_SCROLL);
		tenderPackGrid.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
		tenderPackGrid.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				tenderPackSelectAction();
			}
		});

		GridColumn gridColumn = new GridColumn(tenderPackGrid, SWT.NONE);
		gridColumn.setText(MessageFormat.format(Messages.Title_name, subPackAge));
		gridColumn.setWidth(200);
		gridColumn.setWordWrap(true);

		init();
	}

	/**
	 * @Description: 初始化
	 * @author chenxw
	 */
	private void init() {
		tenderProjectService = new TenderProjectService();
		if (null == tenderProjectEntity) {
			tenderProjectEntity = new TenderProjectEntity();
		}
		List<TenderProjectEntity> tenderPackList = tenderProjectService.getSubPackageList(tenderProjectEntity.getId());
		drawTenderPackTable(tenderPackList);
		tenderPackGrid.setHeaderVisible(true);
		tenderPackGrid.setShowEvenBackground(false);
		tenderPackGrid.setLinesVisible(true);
		tenderPackGrid.setAutoHeight(true);
		tenderPackGrid.setItemHeight(50);
		tenderPackGrid.getVerticalBar().setSelection(tenderPackGrid.getSelectionIndex());
		tenderPackGrid.getVerticalBar().notifyListeners(SWT.Selection, null);
		tenderPackSelectAction();
	}

	/**
	 * @Description: 显示包件列表
	 * @author chenxw
	 * @param tenderPackList
	 */
	private void drawTenderPackTable(final List<TenderProjectEntity> tenderPackList) {
		tenderPackGrid.removeAll();
		if (null != tenderPackList && tenderPackList.size() > 0) {
			for (TenderProjectEntity tenderPack : tenderPackList) {
				GridItem gridItem = new GridItem(tenderPackGrid, SWT.NONE);
				gridItem.setFont(SWTResourceManager.getFont("微软雅黑", 11, SWT.NORMAL));
				gridItem.setText(0, "[" + tenderPack.getTenderNo() + "]" + tenderPack.getTenderName());
				gridItem.setData("tenderPack", tenderPack);
			}
		}
		tenderPackGrid.setSelection(0);
	}

	/**
	 * @Description: 选择包件事件
	 * @author chenxw
	 */
	private void tenderPackSelectAction() {
		if (tenderPackGrid.getSelectionCount() > 0) {
			GridItem selectGridItem = tenderPackGrid.getSelection()[0];
			tenderPackGrid.setFocusItem(selectGridItem);
			TenderProjectEntity tenderPack = (TenderProjectEntity) selectGridItem.getData("tenderPack");
			if (null != tenderPack) {
				if (null != bidFileDownloadBySupplierComposite && !bidFileDownloadBySupplierComposite.isDisposed()) {
					bidFileDownloadBySupplierComposite.dispose();
				}
				bidFileDownloadBySupplierComposite = new DecryptBidFileDownloadBySupplierComposite(contentComposite, tenderPack);
			}
			contentComposite.layout();
		}
	}

	@Override
	protected void checkSubclass() {
		// Disable the check that prevents subclassing of SWT components
	}
}
