package com.gx.obe.common.composite;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.runtime.Platform;
import org.eclipse.swt.SWT;
import org.eclipse.swt.core.MessageDialog;
import org.eclipse.swt.core.utils.FileDialogUtils;
import org.eclipse.swt.core.widgets.EButton;
import org.eclipse.swt.core.widgets.ESepator;
import org.eclipse.swt.custom.CLabel;
import org.eclipse.swt.custom.ScrolledComposite;
import org.eclipse.swt.events.ControlAdapter;
import org.eclipse.swt.events.ControlEvent;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;

import com.gx.obe.ColorConstants;
import com.gx.obe.common.file.enumeration.FileConstants;
import com.gx.obe.common.icons.IconsActivator;
import com.gx.obe.message.Messages;
import com.gx.obe.util.utils.FileUtils;
import com.gx.tools.widget.FileOperatorButton;
import com.swtdesigner.ResourceManager;
import com.swtdesigner.SWTResourceManager;

public class DefaultTempleteListComposite extends Composite {

	private Composite machineComposite;
	private ScrolledComposite scrolledComposite;
	private String templatePath;
	private Composite reportItemComposite;
	private EButton delete_all_button;
	private FileOperatorButton.ClickListener deleteListener;
	
	/**
	 * Create the composite.
	 * @param parent
	 * @param style
	 */
	public DefaultTempleteListComposite(Composite parent, int style) {
		super(parent, style);
		GridLayout gridLayout = new GridLayout(1, false);
		gridLayout.verticalSpacing = 0;
		gridLayout.horizontalSpacing = 0;
		gridLayout.marginHeight = 0;
		gridLayout.marginWidth = 0;
		setLayout(gridLayout);
		setBackground(SWTResourceManager.getColor(SWT.COLOR_WHITE));
		setBackgroundMode(SWT.INHERIT_FORCE);
		
		Composite composite = new Composite(this, SWT.NONE);
		GridLayout gl_composite = new GridLayout(3, false);
		composite.setLayout(gl_composite);
		composite.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
		
		CLabel lblNewLabel = new CLabel(composite, SWT.NONE);
		lblNewLabel.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
		lblNewLabel.setText("系统默认模板");
		lblNewLabel.setForeground(ColorConstants.NOTE_TIPS_COLOR);
//		lblNewLabel.setImage(ResourceManager.getPluginImage(IconsActivator.PLUGIN_ID, "icons/television.png"));
		lblNewLabel.setFont(SWTResourceManager.getFont("微软雅黑", 10, SWT.BOLD));
		
		
		EButton button = new EButton(composite, SWT.NONE);
		button.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false, 1, 1));
		button.addMouseListener(new org.eclipse.swt.events.MouseAdapter() {
			@Override
			public void mouseUp(MouseEvent e) {
				addFilePath();
			}
		});
		button.setImage(ResourceManager.getPluginImage(IconsActivator.PLUGIN_ID, "icons/btn/blue/import.png"));
		button.setText("导入模板");
		
		delete_all_button = new EButton(composite, SWT.NONE);
		delete_all_button.setImage(ResourceManager.getPluginImage(IconsActivator.PLUGIN_ID, "icons/btn/blue/delete.png"));
		delete_all_button.addMouseListener(new MouseAdapter() {
			@Override
			public void mouseUp(MouseEvent e) {
				deleteAlltemplate();
			}
		});
		delete_all_button.setText(Messages.Button_deleteAll);
		
		ESepator sepUP = new ESepator(this, SWT.SEPARATOR | SWT.HORIZONTAL);
		sepUP.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false, 1, 1));
		sepUP.setBackground(ColorConstants.SEP_LIGHT_COLOR);
		
		scrolledComposite = new ScrolledComposite(this, SWT.H_SCROLL | SWT.V_SCROLL);
		scrolledComposite.setExpandHorizontal(true);
		scrolledComposite.setExpandVertical(true);
		scrolledComposite.addControlListener(new ControlAdapter() {
			@Override
			public void controlResized(ControlEvent e) {
				
			}
		});
		scrolledComposite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
		
		machineComposite = new Composite(scrolledComposite, SWT.WRAP);
		machineComposite.setBackground(SWTResourceManager.getColor(SWT.COLOR_WHITE));
		scrolledComposite.setContent(machineComposite);
		machineComposite.setLayout(new GridLayout(1, true));
//		machineComposite.setSize(machineComposite.computeSize(SWT.DEFAULT, SWT.DEFAULT));
//		machineComposite.setLayout(new RowLayout(SWT.HORIZONTAL));
		
		
		reportItemComposite = new Composite(machineComposite, SWT.NONE);
		RowLayout rl_reportItemComposite = new RowLayout(SWT.HORIZONTAL);
		rl_reportItemComposite.spacing = 5;
		rl_reportItemComposite.pack = false;
		reportItemComposite.setLayout(rl_reportItemComposite);
		GridData gd_reportItemComposite = new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1);
		
		
		gd_reportItemComposite.widthHint = getShell().getBounds().x;
		reportItemComposite.setLayoutData(gd_reportItemComposite);
		init();

	}
	
	private void init(){
		deleteListener = new FileOperatorButton.ClickListener() {
			@Override
			public void click(File tenderTemplate, FileOperatorButton fileButton) {
				
				List<File> projectTemplateList = new ArrayList<File>();
				projectTemplateList.add(tenderTemplate);
				deleteTempleteFile(projectTemplateList);
				Composite parent = fileButton.getParent();
				fileButton.dispose();
				parent.layout();
				
			}
		};
		File templateFile = new File(Platform.getInstallLocation().getURL().getPath()+"/template");
		templatePath = templateFile.getAbsolutePath();
		//刷新模板列表
		refreshTemplate();
	}

	/**
	 * @Description: 导入模板保存数据库
	 * @author guoyr
	 */
	public void importTemplate(String filePath){
		
		
		String[] paths = filePath.split(";");
		//遍历选择的各个路径文件模板，保存数据库
		for(String path : paths){
			File file = new File(path);
			FileUtils.copy(file.getAbsolutePath(), templatePath + File.separator + file.getName());
		}
		refreshTemplate();//刷新模板面板
	}
	
	/**
	 * @Description:加载模板组列表 
	 * @author guoyr
	 * @param projectTemplate
	 */
	private void refreshTemplate(){
		//销毁当前面板所有模板，重新加载
		for(Control control : reportItemComposite.getChildren()){
			control.dispose();
		}
		List<File> defaultTemplateFiles = FileUtils.listAllFiles(templatePath);
		if(null != defaultTemplateFiles && defaultTemplateFiles.size() > 0){
			for(File templateFile: defaultTemplateFiles){
				createReportComposite(reportItemComposite, templateFile);
			}
			
		}
		if(null != defaultTemplateFiles && defaultTemplateFiles.size() < 1){
			delete_all_button.setEnabled(false);
		}else{
			delete_all_button.setEnabled(true);
		}
		reSizeFram();
	}
	
	/**
	 * @Description: 弹出选择文件对话框选择文件
	 * @author guoyr
	 */
	public void addFilePath(){
		String filPath = FileDialogUtils.getFileNames(SWT.MULTI, "请选择投标文件(可多选)", new String[]{"*.xml;*.doc;*docx;*.xls;*.xlsx;*.pdf","*.doc;*docx","*.pdf", "*.xml", FileConstants.BID_PRICE_XML}, new String[]{});
		if(null != filPath && !"".equals(filPath)){
			importTemplate(filPath);
		}
		
	}
	/**
	 * @Description: 初始化各个模板的面板类
	 * @author guoyr
	 * @param reportGroupComposite
	 * @param thisTenderTemplate
	 * @return
	 */
	private void createReportComposite(final Composite reportGroupComposite, final File reportFile){
		FileOperatorButton fileOperatorButton = new FileOperatorButton(reportItemComposite, reportFile);
		fileOperatorButton.addOperator(ResourceManager.getPluginImage(IconsActivator.PLUGIN_ID, "icons/btn/blue/delete.png"), "", Messages.Button_delete, deleteListener);
	}
	
	
	/**
	 * @Description: 删除本地模板文件
	 * @author guoyr
	 * @param templateFileList 
	 */
	public void deleteTempleteFile(List<File> templateFileList){
		for(File projectTemplate : templateFileList){
			if(!projectTemplate.delete()){
				projectTemplate.deleteOnExit();
			}
		}
	}

	/**
	 * @Description: 刷新容器
	 * @author guoyr
	 */
	private void reSizeFram(){
		reportItemComposite.layout();
		int itemX = 0;
		int itemY = 0;
		Control[] controls = reportItemComposite.getChildren();
		for(Control control : controls){
			Composite composite = (Composite)control;
			if(itemX < composite.getBounds().width){
				itemX = composite.getBounds().width;
			}
			if(itemY < composite.getBounds().height){
				itemY = composite.getBounds().height;
			}
			
		}
		
		machineComposite.layout();
		scrolledComposite.setMinSize(machineComposite.computeSize(SWT.DEFAULT, SWT.DEFAULT));
		scrolledComposite.setMinHeight(itemY * (controls.length/(getShell().getBounds().width /(itemX+10))+2));
		scrolledComposite.forceFocus();
//		scrolledComposite.setOrigin(0, scrolledComposite.getVerticalBar().getMaximum());
		scrolledComposite.layout();
		layout();
	}
	@Override
	protected void checkSubclass() {
	}
	/**
	 * @Description: 删除所有模板
	 * @author guoyr 
	 */
	private void deleteAlltemplate(){
		if(delete_all_button.getEnabled()){
			if(MessageDialog.openConfirm(getShell(), "确认要删除所有模板吗？")){
				FileUtils.delAllFile(templatePath);
				refreshTemplate();//刷新模板面板
			}
		}
	}
}
