package com.gx.obe.common.composite;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.eclipse.swt.SWT;
import org.eclipse.swt.core.widgets.EButton;
import org.eclipse.swt.custom.LineBackgroundEvent;
import org.eclipse.swt.custom.LineBackgroundListener;
import org.eclipse.swt.custom.StyledText;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Scale;
import org.eclipse.swt.widgets.Text;

import com.gx.obe.ColorConstants;
import com.gx.obe.config.ConfigConstants;
import com.gx.obe.config.utils.PropertiesUtils;
import com.gx.obe.listener.ScrollLEDListener;
import com.gx.obe.message.Messages;
import com.gx.obe.util.utils.ObjectUtils;

public class LedConfigComposite extends Composite {

	private ScrollLEDListener scrollLEDListener;
	private Scale stepScale;
	private Scale timeScale;
	private StyledText ledTitleScrollValue;
	private String meetingTitle;
	private Button cricleScrollYes;
	private int scrollTime = 200;
	private int scrollStep = 5;
	/**
	 * Create the composite.
	 * @param parent
	 * @param style
	 */
	public LedConfigComposite(Composite parent, String meetingTitle, int style) {
		super(parent, style);
		setLayout(new GridLayout(2, false));
		this.meetingTitle= meetingTitle;

		Label ledScroll = new Label(this, SWT.RIGHT);
		ledScroll.setText(Messages.LedScrollType + "：");
		ledScroll.setLayoutData(new GridData(SWT.FILL, SWT.FILL, false, false, 1, 1));
		
		final Composite ledScrollComposite = new Composite(this, SWT.NONE);
		ledScrollComposite.setLayout(new GridLayout(2, false));
		ledScrollComposite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false, 1, 1));
		
		cricleScrollYes = new Button(ledScrollComposite, SWT.RADIO);
		cricleScrollYes.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				changeCircleScroll(true);
			}
		});
		cricleScrollYes.setSelection("01".equals(PropertiesUtils.getProperty(ConfigConstants.LED_CIRCLE_SCROLL, "01")));
		cricleScrollYes.setText(Messages.LedScrollCyclic);
		
		Button cricleScrollNo = new Button(ledScrollComposite, SWT.RADIO);
		cricleScrollNo.setSelection("00".equals(PropertiesUtils.getProperty(ConfigConstants.LED_CIRCLE_SCROLL, "01")));
		cricleScrollNo.setText(Messages.LedScrollByItem);
		cricleScrollNo.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				changeCircleScroll(false);
			}
		});
		
		Label stepSpeed = new Label(this, SWT.RIGHT);
		stepSpeed.setText(Messages.LedScrollOffset + "：");
		stepSpeed.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false, 1, 1));
		
		Composite stepComposite = new Composite(this, SWT.NONE);
		stepComposite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false, 1, 1));
		GridLayout gl_composite_step = new GridLayout(3, false);
		gl_composite_step.marginWidth = 0;
		gl_composite_step.marginHeight = 0;
		stepComposite.setLayout(gl_composite_step);
		
		Label stepMin = new Label(stepComposite, SWT.NONE);
		stepMin.setText(Messages.Small);
		
		stepScale = new Scale(stepComposite, SWT.NONE);
		GridData gd_stepScale = new GridData(SWT.FILL, SWT.FILL, false, false, 1, 1);
		gd_stepScale.widthHint = 300;
		stepScale.setLayoutData(gd_stepScale);
		
		Label stepMax = new Label(stepComposite, SWT.NONE);
		stepMax.setText(Messages.Big);
		
		
		Label timeSpeed = new Label(this, SWT.RIGHT);
		timeSpeed.setText(Messages.LedScrollSpeech + "：");
		timeSpeed.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false, 1, 1));
		
		Composite timeComposite = new Composite(this, SWT.NONE);
		timeComposite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false, 1, 1));
		GridLayout gl_composite_time = new GridLayout(3, false);
		gl_composite_time.marginWidth = 0;
		gl_composite_time.marginHeight = 0;
		timeComposite.setLayout(gl_composite_time);
		
		Label timeMin = new Label(timeComposite, SWT.NONE);
		timeMin.setText(Messages.Fast);
		
		timeScale = new Scale(timeComposite, SWT.NONE);
		GridData gd_timeScale = new GridData(SWT.FILL, SWT.FILL, false, false, 1, 1);
		gd_timeScale.widthHint = 300;
		timeScale.setLayoutData(gd_timeScale);
		
		Label timeMax = new Label(timeComposite, SWT.NONE);
		timeMax.setText(Messages.Slow);
		
		Label ledTitle = new Label(this, SWT.RIGHT);
		ledTitle.setText(Messages.LedTitle + "：");
		ledTitle.setLayoutData(new GridData(SWT.FILL, SWT.FILL, false, false, 1, 1));
		
		Text ledTitleValue = new Text(this, SWT.BORDER);
		ledTitleValue.setText(ObjectUtils.getObjString(meetingTitle));
		ledTitleValue.setEditable(false);
		ledTitleValue.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false, 1, 1));
		
		Label ledTitleScroll = new Label(this, SWT.WRAP | SWT.RIGHT);
		ledTitleScroll.setText(Messages.LedScrollMessage + "：");
		ledTitleScroll.setLayoutData(new GridData(SWT.FILL, SWT.FILL, false, false, 1, 1));
		
		ledTitleScrollValue = new StyledText(this, SWT.BORDER | SWT.MULTI | SWT.V_SCROLL);
		
		GridData gd_ledTitleScrollValue = new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1);
		gd_ledTitleScrollValue.heightHint = 200;
		gd_ledTitleScrollValue.widthHint = 700;
		ledTitleScrollValue.setLayoutData(gd_ledTitleScrollValue);
		
		Label label = new Label(this, SWT.CENTER);
		label.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false, 2, 1));
		label.setText(Messages.Tips_ledScroll);
		label.setForeground(ColorConstants.NOTE_TIPS_COLOR);
		
		EButton saveBtn = new EButton(this, SWT.NONE|EButton.CSS_BLUE );
		saveBtn.setText(Messages.Button_save);
		saveBtn.setLeftMargin(10);
		saveBtn.setRightMargin(10);
		GridData gd_saveBtn = new GridData(SWT.CENTER, SWT.FILL, true, false, 2, 1);
		gd_saveBtn.heightHint = 30;
		gd_saveBtn.widthHint = 90;
		saveBtn.setLayoutData(gd_saveBtn);
		saveBtn.addMouseListener(new MouseAdapter() {
			public void mouseUp(MouseEvent e) {
				saveAction();
			}
		});
		
		init();
	}
	
	private void init(){
		
		String configContent = ObjectUtils.getObjString(PropertiesUtils.getProperty(ConfigConstants.OPENINGROOM_LED_CONTENT, ""));
		if(null != configContent && configContent.length() > 0){
			ledTitleScrollValue.setText(configContent);
		}
		ledTitleScrollValue.setMarginColor(ColorConstants.SKIN_BG);
		ledTitleScrollValue.addLineBackgroundListener(new LineBackgroundListener() {      
			public void lineGetBackground(LineBackgroundEvent event) {      
				if(ledTitleScrollValue.getLineAtOffset(event.lineOffset) % 2 == 1)      
					event.lineBackground = ColorConstants.ENENT_LINE_COLOR;      
			}      
		});  
		
		scrollTime = ObjectUtils.getIntValue(PropertiesUtils.getProperty(ConfigConstants.LED_SCROLL_TIME, "200"), 200);
		scrollStep = ObjectUtils.getIntValue(PropertiesUtils.getProperty(ConfigConstants.LED_SCROLL_STEP, "5"), 5);
		
		stepScale.setToolTipText(scrollStep +"");
		stepScale.setPageIncrement(1);
		stepScale.setMinimum(1);
		stepScale.setMaximum(10);
		stepScale.setSelection(scrollStep);
		stepScale.addListener(SWT.Selection, new Listener() { 
			public void handleEvent(Event event) { 
				scrollStep = stepScale.getSelection();
				stepScale.setToolTipText(scrollStep +"");
				scrollLEDListener.refreshSpeed(scrollTime, scrollStep);
			} 
		});
		timeScale.setToolTipText(scrollTime +"");
		timeScale.setPageIncrement(20);
		timeScale.setMinimum(20);
		timeScale.setMaximum(500);
		timeScale.setSelection(scrollTime);
		timeScale.addListener(SWT.Selection, new Listener() { 
			public void handleEvent(Event event) { 
				scrollTime = timeScale.getSelection();
				timeScale.setToolTipText(scrollTime +"");
				scrollLEDListener.refreshSpeed(scrollTime, scrollStep);
			} 
		});
	}
	
	private void changeCircleScroll(boolean isCircle){
		scrollLEDListener.setCircleScroll(isCircle);
	}
	
	private void saveAction(){
		scrollLEDListener.refreshSpeed(scrollTime, scrollStep);
		
		PropertiesUtils.setPropertyValue(ConfigConstants.LED_SCROLL_TIME, scrollTime+"");
		PropertiesUtils.setPropertyValue(ConfigConstants.LED_SCROLL_STEP, scrollStep+"");
		PropertiesUtils.setPropertyValue(ConfigConstants.LED_CIRCLE_SCROLL, cricleScrollYes.getSelection()? "01" : "00");
		PropertiesUtils.saveProperty(ConfigConstants.OPENINGROOM_LED_CONTENT, ledTitleScrollValue.getText().trim());
		getShell().close();
		if(null != scrollLEDListener){
			String ledContent = PropertiesUtils.getProperty(ConfigConstants.OPENINGROOM_LED_CONTENT, "");
			List<String> ledContents = new ArrayList<String>();
			ledContents.add(meetingTitle);
			if(null != ledContent && ledContent.length() > 0){
				String[] titles = ledContent.split("\r\n");
				ledContents.addAll(Arrays.asList(titles));
			}
			scrollLEDListener.setCircleScroll("01".equals(PropertiesUtils.getProperty(ConfigConstants.LED_CIRCLE_SCROLL, "01")));
			scrollLEDListener.refreshLEDContent(ledContents);
		}
	
	}

	public void addScrollLEDListener(ScrollLEDListener scrollLEDListener){
		this.scrollLEDListener = scrollLEDListener;
	}
	@Override
	protected void checkSubclass() {
		// Disable the check that prevents subclassing of SWT components
	}

}
