package com.gx.obe.common.composite;

import java.text.MessageFormat;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.DisposeEvent;
import org.eclipse.swt.events.DisposeListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;

import com.gx.obe.ColorConstants;
import com.gx.obe.action.bean.Action;
import com.gx.obe.action.listener.ActionListener;
import com.gx.obe.action.utils.ActionUtil;
import com.gx.obe.business.vo.Audit;
import com.gx.obe.common.icons.IconsActivator;
import com.gx.obe.common.shell.MeetingRoomOpenShell;
import com.gx.obe.components.core.Constants;
import com.gx.obe.components.core.enumeration.CommonEnum;
import com.gx.obe.components.core.enumeration.EvaluationFactorEnum;
import com.gx.obe.evaluation.step.listener.ExpertEvaluationListener;
import com.gx.obe.evaluation.step.review.action.PreEvaluationAction;
import com.gx.obe.message.Messages;
import com.gx.obe.web.entity.auth.AuthUser;
import com.gx.obe.web.entity.evaluation.EvaluationStep;
import com.gx.obe.web.entity.tender.TenderProjectEntity;
import com.gx.obe.web.entity.tender.TenderProjectRuleEntity;
import com.gx.obe.web.service.ProjectRuleService;
import com.gx.obe.web.service.expert.ExpertEvaluationStepService;
import com.swtdesigner.ResourceManager;
import com.swtdesigner.SWTResourceManager;

import de.kupzog.ktable.KTable;
import de.kupzog.ktable.SWTX;
import de.kupzog.ktable.editors.KTableCellEditorLink;
import de.kupzog.ktable.model.CommonKtableCModel;
import de.kupzog.ktable.renderers.CellAlignmentRender;
import de.kupzog.ktable.renderers.CellContentRender;
import de.kupzog.ktable.renderers.CellEditorRender;
import de.kupzog.ktable.renderers.CellEventActionRender;
import de.kupzog.ktable.renderers.CellForegroundRender;
import de.kupzog.ktable.renderers.CellImageRender;
import de.kupzog.ktable.renderers.CellRender;
import de.kupzog.ktable.renderers.FixedWrapCellRenderer;
import de.kupzog.ktable.renderers.LinkCellRenderer;
import de.kupzog.ktable.renderers.TextCellRenderer;

public class MyAuditComposite extends Composite implements ActionListener {
    
    private ProjectRuleService projectRuleService = new ProjectRuleService();
    private ExpertEvaluationStepService expertEvaluationStepService = new ExpertEvaluationStepService();
    private TenderProjectEntity curTender;
    private MeetingRoomOpenShell meetingRoomOpenShell;
    private Map<String, String> stepStatusMap;
    private Map<String, String> operateTypeMap;
    private Composite tableComposite;
    private KTable auditTable;
    private String stepStatus;
    
    /**
     * Create the composite.
     * 
     * @param parent
     * @param style
     */
    public MyAuditComposite(Composite parent, int style) {
        super(parent, style);
        setLayout(new GridLayout(1, false));
        
        Composite topComposite = new Composite(this, SWT.NONE);
        topComposite.setBackground(ColorConstants.BORDER_LINE_COLOR);
        GridLayout gl_topComposite = new GridLayout(1, false);
        gl_topComposite.marginWidth = 1;
        gl_topComposite.marginHeight = 1;
        topComposite.setLayout(gl_topComposite);
        topComposite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
        
        tableComposite = new Composite(topComposite, SWT.NONE);
        tableComposite.setBackground(SWTResourceManager.getColor(SWT.COLOR_WHITE));
        GridLayout gl_composite = new GridLayout(1, false);
        gl_composite.marginHeight = 0;
        gl_composite.marginWidth = 0;
        tableComposite.setLayout(gl_composite);
        tableComposite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
        
    }
    
    public void refresh(TenderProjectEntity curTender, String stepStatus) {
        this.curTender = curTender;
        this.stepStatus = stepStatus;
        refreshStepStatusMap();
        refreshOperateTypeMap();
        refreshAuditTable();
    }
    
    private void refreshStepStatusMap() {
        stepStatusMap = new HashMap<String, String>();
        stepStatusMap.put(CommonEnum.IN_PROGRESS, Messages.Progress_inProgress);
        stepStatusMap.put(CommonEnum.COMPLETE, Messages.Progress_auditComplate);
        stepStatusMap.put(CommonEnum.IN_EXPERT_REVIEW, MessageFormat.format(Messages.Progress_pendingAuditFormat, Messages.ExpertLeader));
        stepStatusMap.put(CommonEnum.IN_MANAGER_REVIEW, MessageFormat.format(Messages.Progress_pendingAuditFormat, Messages.ProjectManager));
    }
    
    private void refreshOperateTypeMap() {
        operateTypeMap = new HashMap<String, String>();
//        operateTypeMap.put(EvaluationFactorEnum.EVAL_BID_DETAIL, "审核清标意见");
//        operateTypeMap.put(EvaluationFactorEnum.EVAL_BID_BUS, "审核商务清标意见");
//        operateTypeMap.put(EvaluationFactorEnum.EVAL_BID_TEC, "审核技术清标意见");
//        operateTypeMap.put(EvaluationFactorEnum.EVAL_BID_PRICE, "审核报价对比");
        operateTypeMap.put(EvaluationFactorEnum.EVAL_BID_DETAIL, Messages.Button_approval);
        operateTypeMap.put(EvaluationFactorEnum.EVAL_BID_BUS, Messages.Button_approval);
        operateTypeMap.put(EvaluationFactorEnum.EVAL_BID_TEC, Messages.Button_approval);
        operateTypeMap.put(EvaluationFactorEnum.EVAL_BID_PRICE, Messages.Button_approval);
    }
    
    private void refreshAuditTable() {
        if (null != auditTable) {
            auditTable.dispose();
        }
        
        auditTable = new KTable(tableComposite, SWTX.AUTO_SCROLL | SWT.FULL_SELECTION | SWTX.FILL_WITH_LASTCOL | SWTX.EDIT_ON_KEY);
        auditTable.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
        final List<Audit> auditList = expertEvaluationStepService.getAuditListByStepType(curTender.getId(), com.gx.obe.components.core.enumeration.EvaluationStepEnum.PRE_EVALUATION);
        
        final CommonKtableCModel model = new CommonKtableCModel(auditTable, 1, 0, auditList.size(), 5);
        auditTable.setModel(model);
        
        //设置行高
        final int indexCol = 0;
        final int stepNameCol = 1;
        final int userNameCol = 2;
        final int stepStatusCol = 3;
        final int operateTypeCol = 4;
        
        model.setContentRowHeight(Constants.ROW_HIGHT);
        model.setColumnWidth(0, 10);
        model.setColumnMaxWidth(0, 50);
        model.setColumnWidth(stepNameCol, 150);
        model.setColumnWidth(userNameCol, 50);
        model.setColumnWidth(stepStatusCol, 120);
        model.setColumnWidth(operateTypeCol, 50);
        
        
        model.setCellContentRender(new CellContentRender() {
            public Object receiveContent(int row, int col) {
                if (row < model.getFixedRowCount()) {
                    switch (col) {
                        case indexCol:
                            return Messages.Num;
                        case stepNameCol:
                            return Messages.AuditContent;
                        case userNameCol:
                            return Messages.Submitter;
                        case stepStatusCol:
                            return Messages.Status;
                        case operateTypeCol:
                            return Messages.Operation;
                    }
                } else {
                    int auditIndex = row - model.getFixedRowCount();
                    Audit audit = auditList.get(auditIndex);
                    switch (col) {
                        case indexCol:
                            return auditIndex + 1;
                        case stepNameCol:
                            return audit.getEvaluationStep().getStepName();
                        case userNameCol:
                            return audit.getUser().getUserName();
                        case stepStatusCol:
                            String stepStatus = audit.getExpertEvaluationStep().getStepStatus();
                            if (stepStatusMap.containsKey(stepStatus)) {
                                return stepStatusMap.get(stepStatus);
                            }
                            return Messages.Progress_noStart;
                        case operateTypeCol:
                            String evalBidType = audit.getEvaluationStep().getEvalBidType();
                            if (operateTypeMap.containsKey(evalBidType)) {
                                return operateTypeMap.get(evalBidType);
                            }
                            return "";
                    }
                }
                return "";
            }
        });
        final LinkCellRenderer linkCellRenderer = new LinkCellRenderer();
        model.setCellRender(new CellRender() {
            public Object receiveCellRender(int row, int col) {
            	if (model.isFixedCell(col, row)) {
					return new FixedWrapCellRenderer();
				}
                if (row >= model.getFixedRowCount()) {
                    switch (col) {
                        case operateTypeCol:
                            Audit audit = auditList.get(row - model.getFixedRowCount());
                            linkCellRenderer.setEnable(stepStatus.equals(audit.getExpertEvaluationStep().getStepStatus()));
                            return linkCellRenderer;
                    }
                }
                return new TextCellRenderer();
            }
        });
        
        model.setCellEditorRender(new CellEditorRender() {
            public Object receiveCellEditor(int row, int col) {
                if (row >= model.getFixedRowCount()) {
                    switch (col) {
                        case operateTypeCol:
                            return new KTableCellEditorLink(auditTable, new CellEventActionRender() {
                                public void receiveAction(int row, int col) {
                                    Audit audit = auditList.get(row - model.getFixedRowCount());
                                    auditAction(audit.getEvaluationStep(), audit.getUser());
                                }
                            });
                    }
                }
                return null;
            }
        });
        
        // 设置单元格图片
        final Image editImage = ResourceManager.getPluginImage(IconsActivator.PLUGIN_ID, "icons/btn/blue/signature.png");
        model.setCellImageRender(new CellImageRender() {
            public Image receiveCellImage(int row, int col) {
                if (row >= model.getFixedRowCount()) {
                    switch (col) {
                        case operateTypeCol:
                            return editImage;
                    }
                }
                return null;
            }
        });
        
        // 设置单元格前景颜色
        model.setCellForegroundRender(new CellForegroundRender() {
            public Color receiveCellForeground(int row, int col) {
                if (row >= model.getFixedRowCount()) {
                    switch (col) {
                        case operateTypeCol:
                            return ColorConstants.SKIN_BG;
                    }
                }
                return null;
            }
        });
        
        // 编辑单元格对齐方式
        model.setCellAlignmentRender(new CellAlignmentRender() {
            public int receiveCellAlignment(int row, int col) {
                if (row >= model.getFixedRowCount()) {
                    switch (col) {
                        case stepNameCol:
                            return SWTX.ALIGN_VERTICAL_CENTER | SWTX.ALIGN_HORIZONTAL_LEFT;
                    }
                }
                return SWTX.ALIGN_VERTICAL_CENTER | SWTX.ALIGN_HORIZONTAL_CENTER;
            }
        });
        
        tableComposite.layout();
        auditTable.resizeColumnOptimal(stepStatusCol);
    }
    
    private void auditAction(EvaluationStep evaluationStep, AuthUser user) {
        
        Action action = ActionUtil.getAction(PreEvaluationAction.class.getName());
        action.addActionListener(this);
        action.setData("expertEvaluationListener", new ExpertEvaluationListener() {
            public void complete() {
                if (null != meetingRoomOpenShell && !meetingRoomOpenShell.isDisposed()) {
                    meetingRoomOpenShell.close();
                    refreshAuditTable();
                }
            }
            
            public void refresh() {
                refreshAuditTable();
            }
        });
        action.setTenderProject(curTender);
        action.setData("evaluationStep", evaluationStep);
        action.setData("curUser", user);
        action.run();
    }
    
    public void onLoad(Action action) {
        action.setData("tenderProject", curTender);
        
        TenderProjectRuleEntity projectRule = projectRuleService.getProjectRule(curTender.getId());
        if (null == projectRule) {
            projectRule = new TenderProjectRuleEntity();
        }
        
        action.setData("projectRule", projectRule);
        if (stepStatus.equals(CommonEnum.IN_MANAGER_REVIEW)) {
            action.setData("hideLeft", "true");
        }
        
        meetingRoomOpenShell = new MeetingRoomOpenShell(getShell(), curTender);
        meetingRoomOpenShell.setMaximized(false);
        meetingRoomOpenShell.addDisposeListener(new DisposeListener() {
            public void widgetDisposed(DisposeEvent e) {
                refreshAuditTable();
            }
        });
        meetingRoomOpenShell.open();
    }
    
    public void loadComposite(Action action, Composite composite) {
        meetingRoomOpenShell.getParentComposite().layout();
    }
    
    public Composite getParentComposite() {
        return meetingRoomOpenShell.getParentComposite();
    }
    
    @Override
    protected void checkSubclass() {
        // Disable the check that prevents subclassing of SWT components
    }
    
}
