package com.gx.obe.common.composite;

import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.List;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;

import com.gx.obe.components.core.I18NEnum;
import com.gx.obe.components.core.Constants;
import com.gx.obe.components.core.enumeration.CommonEnum;
import com.gx.obe.message.Messages;
import com.gx.obe.util.utils.LogUtils;
import com.gx.obe.util.utils.ObjectUtils;
import com.gx.obe.web.entity.Supplier;
import com.gx.obe.web.entity.SupplierEvaluationResult;
import com.gx.obe.web.entity.SupplierOpeningResult;
import com.gx.obe.web.entity.tender.TenderProjectEntity;
import com.gx.obe.web.entity.tender.TenderProjectRuleEntity;
import com.gx.obe.web.service.ProjectRuleService;
import com.gx.obe.web.service.SupplierService;
import com.swtdesigner.SWTResourceManager;

import de.kupzog.ktable.KTable;
import de.kupzog.ktable.SWTX;
import de.kupzog.ktable.model.CommonKtableCModel;
import de.kupzog.ktable.renderers.CellAlignmentRender;
import de.kupzog.ktable.renderers.CellContentRender;
import de.kupzog.ktable.renderers.CellForegroundRender;
import de.kupzog.ktable.renderers.CellRender;
import de.kupzog.ktable.renderers.FixedCellRenderer;
import de.kupzog.ktable.renderers.TextWrapCellRenderer;

/**
 * @Description: 
 * @author guoyr
 */
public class OpinionInputResultComposite extends Composite {

	private Composite tableComposite;
	private KTable resultKTable;
	private CommonKtableCModel commKTableModel;
	private TenderProjectEntity tenderProjectEntity;
	private TenderProjectRuleEntity tenderProjectRuleEntity;
	private ProjectRuleService projectRuleService;
	private SupplierService supplierService = new SupplierService();
	private List<Supplier> supplierList;
	private boolean inputEvalPrice = true;
	private String evaluationOpinion = "";
	
	{
		LogUtils.logClass(OpinionInputResultComposite.class);
	}
	
	/**
	 * Create the composite.
	 * @param parent
	 * @param style
	 */
	public OpinionInputResultComposite(Composite parent, int style) {
		super(parent, style);
		this.tenderProjectEntity = (TenderProjectEntity)parent.getData("tenderProjectEntity");
		GridLayout gridLayout = new GridLayout(1, false);
		gridLayout.verticalSpacing = 0;
		gridLayout.horizontalSpacing = 0;
		gridLayout.marginHeight = 1;
		gridLayout.marginWidth = 1;
		setLayout(gridLayout);
		setBackground(SWTResourceManager.getColor(211, 211, 211));
		Composite parentComposite = new Composite(this, SWT.NONE);
		GridLayout gl_parentComposite = new GridLayout(1, false);
		gl_parentComposite.verticalSpacing = 0;
		gl_parentComposite.horizontalSpacing = 0;
		gl_parentComposite.marginHeight = 0;
		gl_parentComposite.marginWidth = 0;
		parentComposite.setLayout(gl_parentComposite);
		parentComposite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));	
		
		parentComposite.setBackground(SWTResourceManager.getColor(SWT.COLOR_WHITE));
		tableComposite = new Composite(parentComposite, SWT.NONE);
		tableComposite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
		tableComposite.setLayout(new FillLayout(SWT.HORIZONTAL));

		init();
	}

	/**
	 * @Description: 初始化
	 * @author guoyr
	 */
	private void init(){
		projectRuleService = new ProjectRuleService();
		tenderProjectRuleEntity = projectRuleService.getProjectRule(tenderProjectEntity.getId());
		if(null == tenderProjectRuleEntity){
			tenderProjectRuleEntity = new TenderProjectRuleEntity();
		}
		inputEvalPrice = ObjectUtils.equals(tenderProjectRuleEntity.getInputEvalPrice(), CommonEnum.YES);
		drawTable();
	}

	/**
	 * @Description: 画评审意见表格
	 * @author guoyr
	 */
	private void drawTable(){
		supplierList = supplierService.getSupplierOpenbidAndEvaluateResultList(tenderProjectEntity.getId());
		resultKTable = new KTable(tableComposite, SWTX.AUTO_SCROLL | SWT.FULL_SELECTION | SWTX.EDIT_ON_KEY | SWTX.FILL_WITH_LASTCOL);
		if(null == supplierList || supplierList.size() == 0){
			supplierList = new ArrayList<Supplier>();
		}
		commKTableModel = new CommonKtableCModel(resultKTable, 1, 1, supplierList.size() + 1 , 1 + (inputEvalPrice?1:0));
		final String[][]contentArray = new String[commKTableModel.getRowCount()][commKTableModel.getColCount()];
		contentArray[0][0] = com.gx.obe.components.core.I18N.getString(MessageFormat.format(Messages.NameFormat, com.gx.obe.components.core.I18N.getString(I18NEnum.SUPPLIERNAME, tenderProjectEntity.getTenderType())));
		contentArray[0][1] = Messages.Title_evaluationOpinion;
		if(inputEvalPrice){
			commKTableModel.setColumnMaxWidth(1, 150);
			commKTableModel.setColumnWidth(2, 300);
			contentArray[0][1] = Messages.EvaluationPrice+"（"+tenderProjectRuleEntity.getPriceUnitUseDefault()+"）";
			contentArray[0][2] = Messages.Title_evaluationOpinion;
		}else {
			commKTableModel.setColumnWidth(1, 300);
		}
		contentArray[commKTableModel.getRowCount()-1][0] = "评审结论";
		evaluationOpinion = null != tenderProjectEntity.getEvaluationOpinion()?tenderProjectEntity.getEvaluationOpinion()+"  ":"";
		commKTableModel.addColSpan(commKTableModel.getRowCount()-1, 1, commKTableModel.getContentCol());
		int row = 0;
		commKTableModel.setContentRowHeight(30);
		
		// 如果匿名显示
		boolean isAnonymous = false;
		// 如果显示匿名，验证是否要匿名
		if(Constants.SHOW_ANONYMOUS){
			isAnonymous = projectRuleService.isAnonymous(tenderProjectEntity.getId());
		}
		int index = 1;
		for(Supplier supplier : supplierList){
			supplier.setAnonymous(isAnonymous, index ++);
			contentArray[row + commKTableModel.getFixedRowCount()][0] = supplier.getSupplierName();
			SupplierEvaluationResult evaluationResult = supplier.getSupplierEvaluationResult();
			if(null == evaluationResult){
				supplier.setSupplierEvaluationResult(new SupplierEvaluationResult());
			}
			SupplierOpeningResult openbidResult = supplier.getSupplierOpeningResult();
			if(null == openbidResult){
				supplier.setSupplierOpeningResult(new SupplierOpeningResult());
			}
			row ++;
		}
		// 设置单元格内容
		CellContentRender contentRender = new CellContentRender(){
			public Object receiveContent(int row, int col) {
				if(resultKTable.isFixedCell(col, row)){
					if(null != contentArray[row][col]){
						return contentArray[row][col];
					}
				}else if(row < commKTableModel.getRowCount() -1){
					Supplier supplier =  supplierList.get(row - commKTableModel.getFixedRowCount());
					if(inputEvalPrice && col == 1){
						SupplierOpeningResult openbidResult = supplier.getSupplierOpeningResult();
						if(null != openbidResult.getEvaluationPrice()){
							return openbidResult.getFormatEvaluatoinPriceByUnit(tenderProjectRuleEntity.getEvalPriceUnit());
						}
					}else {
						SupplierEvaluationResult evaluationResult = supplier.getSupplierEvaluationResult();
						if(null != evaluationResult.getEvaluationOpinion()){
							return evaluationResult.getEvaluationOpinion();
						}
					}
				}else {
					return evaluationOpinion;
				}
				return "";
		}};
		commKTableModel.setCellContentRender(contentRender);
		
		// 设置单元格渲染器
		CellRender cellRender = new CellRender(){
			public Object receiveCellRender(int row, int col) {
				if(resultKTable.isFixedCell(col, row)){
					return new FixedCellRenderer();
				}else{
					return new TextWrapCellRenderer();
				}
		}};
		commKTableModel.setCellRender(cellRender);
		
		// 编辑单元格对齐方式
		CellAlignmentRender cellAlignmentRender = new CellAlignmentRender(){
			public int receiveCellAlignment(int row, int col) {
				if(resultKTable.isFixedCell(col, row)){
					if(row > 0 && row < commKTableModel.getRowCount() -1){
						return SWTX.ALIGN_VERTICAL_CENTER | SWTX.ALIGN_HORIZONTAL_LEFT;
					}else {
						return SWTX.ALIGN_VERTICAL_CENTER | SWTX.ALIGN_HORIZONTAL_CENTER;
					}
				}else {
					if(inputEvalPrice && col == 1 && row < commKTableModel.getRowCount() -1){
						return SWTX.ALIGN_VERTICAL_CENTER | SWTX.ALIGN_HORIZONTAL_RIGHT;
					}else {
						return SWTX.ALIGN_VERTICAL_CENTER | SWTX.ALIGN_HORIZONTAL_LEFT;
					}
				}
			}
		};
		commKTableModel.setCellAlignmentRender(cellAlignmentRender);
		
		CellForegroundRender cellForegroundRender = new CellForegroundRender(){
			public Color receiveCellForeground(int row, int col) {
				if(commKTableModel.isFixedCell(col, row)){
					if(col == 0 && row < commKTableModel.getRowCount() -1){
						Supplier supplier =  supplierList.get(row - commKTableModel.getFixedRowCount());
						SupplierEvaluationResult evaluationResult = supplier.getSupplierEvaluationResult();
						if(ObjectUtils.equals(evaluationResult.getAccessment(), com.gx.obe.components.core.enumeration.EvaluationStepEnum.UNQUALIFIED)){
							return SWTResourceManager.getColor(SWT.COLOR_RED);
						}
					}
				}
				return null;
		}};
		commKTableModel.setCellForegroundRender(cellForegroundRender);
		resultKTable.setModel(commKTableModel);
	}
	
	@Override
	protected void checkSubclass() {
		// Disable the check that prevents subclassing of SWT components
	}

}
