package com.gx.obe.common.composite;

import org.eclipse.swt.SWT;
import org.eclipse.swt.core.widgets.EButton;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;

import com.gx.obe.ColorConstants;
import com.swtdesigner.SWTResourceManager;

public class PaginationComposite extends Composite {
	
	private EButton homePage, previousPage, nextPage, lastPage;
	
	private Text pageText;
	
	private Label pageCountLabel;
	
	private PageListener listener;
	
	private int pageCount = 1, pageNumber = 1, page = 1;
	
	/**
	 * Create the composite.
	 * 
	 * @param parent
	 * @param style
	 */
	public PaginationComposite(Composite parent, int style) {
		super(parent, style);
		setLayout(new GridLayout(6, false));
		
		homePage = creageButton(this);
		homePage.setText("首页");
		homePage.addMouseListener(new MouseAdapter() {
			public void mouseUp(MouseEvent e) {
				pageNumber = 1;
				skipPage();
			}
		});
		
		previousPage = creageButton(this);
		previousPage.setText("上一页");
		previousPage.addMouseListener(new MouseAdapter() {
			public void mouseUp(MouseEvent e) {
				if (pageNumber > 1) {
					pageNumber--;
				}
				skipPage();
			}
		});
		
		nextPage = creageButton(this);
		nextPage.setText("下一页");
		nextPage.addMouseListener(new MouseAdapter() {
			public void mouseUp(MouseEvent e) {
				if (pageNumber < pageCount) {
					pageNumber++;
				}
				skipPage();
			}
		});
		
		lastPage = creageButton(this);
		lastPage.setText("末页");
		lastPage.addMouseListener(new MouseAdapter() {
			public void mouseUp(MouseEvent e) {
				pageNumber = pageCount;
				skipPage();
			}
		});
		
		Composite composite = new Composite(this, SWT.NONE);
		GridLayout gl_composite = new GridLayout(2, false);
		gl_composite.horizontalSpacing = 0;
		composite.setLayout(gl_composite);
		
		pageText = new Text(composite, SWT.CENTER | SWT.BORDER);
		GridData gd_text = new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1);
		gd_text.widthHint = 50;
		pageText.setLayoutData(gd_text);
		pageText.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent arg0) {
				try {
					page = Integer.parseInt(pageText.getText().toString().trim());
					if (page < 1 || page > pageCount) {
						pageText.setText("1");
					}
				} catch (Exception e) {
					pageText.setText("1");
				}
			}
		});
		
		pageCountLabel = new Label(composite, SWT.NONE);
		
		EButton skipPage = creageButton(this);
		skipPage.setText("跳转");
		skipPage.addMouseListener(new MouseAdapter() {
			public void mouseUp(MouseEvent e) {
				pageNumber = page;
				skipPage();
			}
		});
		
	}
	
	private void skipPage() {
		pageText.setText(pageNumber + "");
		nextPage.setEnabled(true);
		lastPage.setEnabled(true);
		homePage.setEnabled(true);
		previousPage.setEnabled(true);
		if (pageNumber == 1) {
			homePage.setEnabled(false);
			previousPage.setEnabled(false);
		}
		if (pageNumber == pageCount) {
			nextPage.setEnabled(false);
			lastPage.setEnabled(false);
		}
		if (null != listener) {
			listener.gotoPage(pageNumber);
		}
	}
	
	private EButton creageButton(Composite parent) {
		EButton button = new EButton(parent, SWT.NONE);
		button.setForeground(ColorConstants.SKIN_FOREGROUND);
		button.setActiveForeground(ColorConstants.SKIN_ACTIVE_FOREGROUND);
		button.setLeftMargin(10);
		button.setRightMargin(10);
		button.setBorderColor(null);
		button.setActiveBorderColor(null);
		button.setBackgroundColor(ColorConstants.SKIN_BG);
		button.setActiveBgColor(ColorConstants.SKIN_ACTIVE_BG);
		button.setDisenableColor(SWTResourceManager.getColor(210, 210, 210));
		return button;
	}
	
	public void setPageCount(int pageNumber, int pageCount) {
		this.pageNumber = pageNumber > 0 ? pageNumber : 1;
		this.pageCount = pageCount > 0 ? pageCount : 1;
		pageCountLabel.setText("/" + this.pageCount);
		pageCountLabel.getParent().layout();
		skipPage();
	}
	
	public void addPageListener(PageListener listener) {
		this.listener = listener;
	}
	
	public interface PageListener {
		void gotoPage(int pageNumber);
	}
	
	@Override
	protected void checkSubclass() {
		// Disable the check that prevents subclassing of SWT components
	}
	
	public static void main(String args[]) {
		try {
			Display display = Display.getDefault();
			Shell shell = new Shell(display);
			shell.setLayout(new GridLayout(1, false));
			PaginationComposite composite = new PaginationComposite(shell, SWT.NONE);
			composite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
			composite.setPageCount(5, 100);
			shell.open();
			shell.layout();
			while (!shell.isDisposed()) {
				if (!display.readAndDispatch()) {
					display.sleep();
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
}
