package com.gx.obe.common.composite;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.eclipse.nebula.widgets.grid.Grid;
import org.eclipse.nebula.widgets.grid.GridColumn;
import org.eclipse.nebula.widgets.grid.GridItem;
import org.eclipse.nebula.widgets.grid.listener.GridSelectionListener;
import org.eclipse.swt.SWT;
import org.eclipse.swt.core.widgets.ELink;
import org.eclipse.swt.custom.ScrolledComposite;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.swtchart.Chart;
import org.swtchart.ILineSeries;
import org.swtchart.ILineSeries.PlotSymbolType;
import org.swtchart.ISeries.SeriesType;
import org.swtchart.LineStyle;

import com.gx.obe.common.icons.IconsActivator;
import com.gx.obe.common.widget.dialog.SelectFactorDialog;
import com.gx.obe.common.widget.listener.SelectFactorListener;
import com.gx.obe.components.core.I18NEnum;
import com.gx.obe.util.utils.BigDecimalUtils;
import com.gx.obe.web.entity.EvaluationFactorResult;
import com.gx.obe.web.entity.Supplier;
import com.gx.obe.web.entity.evaluation.EvaluationFactor;
import com.gx.obe.web.entity.evaluation.EvaluationStep;
import com.gx.obe.web.entity.expert.Expert;
import com.gx.obe.web.entity.tender.TenderProjectEntity;
import com.gx.obe.web.service.SupplierService;
import com.gx.obe.web.service.evaluation.EvaluationFactorResultService;
import com.gx.obe.web.service.evaluation.EvaluationFactorService;
import com.gx.obe.web.service.evaluation.EvaluationStepService;
import com.gx.obe.web.service.expert.ExpertEvaluationStepService;
import com.swtdesigner.ResourceManager;
import com.swtdesigner.SWTResourceManager;

public class SupplierChartComposite extends Composite {
	private TenderProjectEntity tenderProjectEntity;
	private EvaluationFactorService evaluationFactorService = new EvaluationFactorService();
	private EvaluationStepService evaluationStepService = new EvaluationStepService();
	private SupplierService supplierService = new SupplierService();
	private ExpertEvaluationStepService expertEvaluationStepService ;
	private EvaluationFactorResultService evaluationFactorResultService = new EvaluationFactorResultService();
	private List<EvaluationFactor> evaluationFactorList;
	private EvaluationStep evaluationStep;
	private List<Supplier> supplierList;
	private List<Expert> expertList;
	private String supplierName = "";
	private Grid supplier_grid;
	private Chart chart;
	private Composite composite;
	private Map<String, Double> evaluationFactorResultMap = new HashMap<String, Double>();
	private List<Color> colorList = new ArrayList<Color>();
	private Supplier curSupplier;
	private Color fullScoreLineColor = ResourceManager.getColor(231, 135, 139);
	private Color avgScoreLineColor = ResourceManager.getColor(140, 200, 186);
	private List<EvaluationFactor> selectFactorList;
	private ELink selectEvaluactionFactor_link;
	
	private BigDecimal HightScore = BigDecimal.ZERO;
	private ScrolledComposite scrolledComposite;
	/**
	 * Create the composite.
	 * @param parent
	 * @param style
	 */
	public SupplierChartComposite(Composite parent, int style, TenderProjectEntity tenderProjectEntity) {
		super(parent, style);
		this.tenderProjectEntity = tenderProjectEntity;
		supplierName = com.gx.obe.components.core.I18N.getString(I18NEnum.SUPPLIERNAME, tenderProjectEntity.getTenderType());
		setLayout(new GridLayout(2, false));
		Composite composite_1 = new Composite(this, SWT.NONE);
		composite_1.setLayout(new GridLayout(1, false));
		GridData gd_composite_1 = new GridData(SWT.FILL, SWT.FILL, false, false, 1, 1);
		gd_composite_1.widthHint = 300;
		composite_1.setLayoutData(gd_composite_1);
		
		supplier_grid = new Grid(composite_1, SWT.BORDER);
		supplier_grid.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
		
		GridColumn column = new GridColumn(supplier_grid, SWT.WRAP );
		column.setFooterFont(SWTResourceManager.getFont("微软雅黑", 11, SWT.NORMAL));
		column.setText(supplierName);
		supplier_grid.setHeaderVisible(true);
		Composite composite_2 = new Composite(this, SWT.NONE);
		composite_2.setLayout(new GridLayout(1, false));
		composite_2.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
		
		Composite composite_3 = new Composite(composite_2, SWT.NONE);
		composite_3.setLayout(new GridLayout(1, false));
		composite_3.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, true, false, 1, 1));
		
		selectEvaluactionFactor_link = new ELink(composite_3, SWT.NONE);
		selectEvaluactionFactor_link.addMouseListener(new MouseAdapter() {
			public void mouseUp(MouseEvent e) {
				
				selectEvaluationFactorList();
			}
		});
		selectEvaluactionFactor_link.setImage(ResourceManager.getPluginImage(IconsActivator.PLUGIN_ID, "icons/btn/blue/config.png"));
		selectEvaluactionFactor_link.setText("配置指标");
		selectEvaluactionFactor_link.setEnabled(false);
		scrolledComposite = new ScrolledComposite(composite_2, SWT.BORDER | SWT.H_SCROLL | SWT.V_SCROLL);
		scrolledComposite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
		scrolledComposite.setExpandHorizontal(true);
		scrolledComposite.setExpandVertical(true);
		
		composite = new Composite(scrolledComposite, SWT.NONE);
		composite.setLayout(new FillLayout(SWT.HORIZONTAL));
		composite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
//		composite_4 = new Composite(scrolledComposite, SWT.NONE);
		scrolledComposite.setContent(composite);
		scrolledComposite.setMinSize(composite.computeSize(SWT.DEFAULT, SWT.DEFAULT));
        init();
	}

	@Override
	protected void checkSubclass() {
	}
	
	/**
	 * @Description: 选择评审指标
	 * @author guoyr 
	 */
	private void selectEvaluationFactorList(){
		if(selectEvaluactionFactor_link.getEnabled()){
			SelectFactorDialog dialog = new SelectFactorDialog(getShell(), evaluationFactorList, null, true, new SelectFactorListener() {
				public void setEvaluationFactorList(
						List<EvaluationFactor> evaluationFactorList) {
					selectFactorList = evaluationFactorList;
					loadChartList(curSupplier, selectFactorList);
				}
			});
			dialog.open();
		}
	}
	/**
	 * @Description: 初始化
	 * @author guoyr 
	 */
	private void init(){
		expertEvaluationStepService = new ExpertEvaluationStepService();
		setColor();
		
		List<EvaluationStep> evaluaitonStepList = evaluationStepService.getEvaluationStepListByType(tenderProjectEntity.getId(), com.gx.obe.components.core.enumeration.EvaluationStepEnum.DETAIL);
        
        if (evaluaitonStepList != null && evaluaitonStepList.size() == 1) {
            evaluationStep = evaluaitonStepList.get(0);
        }   
		
		supplierList = supplierService.getEvaluationStepSupplierList(tenderProjectEntity.getId(), evaluationStep);
		expertList = expertEvaluationStepService.getExpertListByStepFinish(tenderProjectEntity.getId(), evaluationStep.getId());
		setFactorList();
		selectFactorList = evaluationFactorList;
		// 添选中事件和右键菜单事件
		supplier_grid.addListener(SWT.Selection, new GridSelectionListener(){
			@Override
			public void leftMouseButtonDown(GridItem item) {
				Supplier supplier = (Supplier)item.getData("supplier");
				if(null != supplier){
					curSupplier = supplier;
					loadChartList(supplier, selectFactorList);
				}else{
					selectEvaluactionFactor_link.setEnabled(false);
				}
			}
			@Override
			public void rightMouseButtonDown(GridItem item) {
			}
		});
		setDataMap();
		reflashSupplierGrid();
	}
	
	/**
	 * @Description: 刷新供应商信息
	 * @author guoyr 
	 */
	private void reflashSupplierGrid(){
		if(null != supplierList && supplierList.size() > 0){
			for(Supplier supplier : supplierList){
				GridItem item = new GridItem(supplier_grid, SWT.WRAP);
				item.setFont(SWTResourceManager.getFont("微软雅黑", 11, SWT.NORMAL));
				item.setText(0, supplier.getSupplierName());
				item.setData("supplier", supplier);
			}
			supplier_grid.setSelection(0);
			curSupplier = supplierList.get(0);
			loadChartList(curSupplier, selectFactorList);
		}
	}
	/**
	 * @Description: 设置样式颜色
	 * @author guoyr 
	 */
	private void setColor(){
		colorList.add(ResourceManager.getColor(255, 0, 0 ));
		colorList.add(ResourceManager.getColor(0, 255, 0));
		colorList.add(ResourceManager.getColor(0, 0, 255));
		colorList.add(ResourceManager.getColor(255, 255, 0));
		colorList.add(ResourceManager.getColor(255, 0, 255));
		colorList.add(ResourceManager.getColor(0, 255, 255));
		colorList.add(ResourceManager.getColor(0, 0, 0));
	}
	
	/**
	 * @Description: 
	 * @author guoyr
	 * @param suplier
	 * @param factorList 
	 */
	private void loadChartList(Supplier suplier, List<EvaluationFactor> factorList){
		selectEvaluactionFactor_link.setEnabled(true);
		if(null != chart && !chart.isDisposed()){
			chart.dispose();
		}
		chart = new Chart(composite, SWT.NONE);
		chart.getTitle().setText("[" + suplier.getSupplierName()  + "] 评分分析离散图");
		chart.getAxisSet().getYAxis(0).getTitle().setText("得分");
	    chart.getAxisSet().getXAxis(0).getTitle().setText("指标项");
		chart.getAxisSet().getXAxis(0).enableCategory(true);
        chart.getAxisSet().getXAxis(0).setCategorySeries(getFactors(factorList));
        chart.setBackgroundMode(SWT.INHERIT_FORCE);
        chart.getAxisSet().getXAxis(0).getTick().setTickLabelAngle(45);
        Integer[] firstValue = new Integer[]{255, 100, 0};
        Integer[] secondValue = new Integer[]{0, 255, 0};
        Integer[] thirdValue = new Integer[]{0, 0, 255};
        double[] avgSeries = new double[factorList.size()];//计算平均分
        int expertIndex = 0;
        int size = 40;
		for(Expert expert : expertList){
			double[] ySeries = new double[factorList.size()];
			int index = 0;
			for(EvaluationFactor factor : factorList){

				if(null != evaluationFactorResultMap.get(suplier.getId() + factor.getId() + expert.getUser().getId())){
					ySeries[index] = evaluationFactorResultMap.get(suplier.getId() + factor.getId() + expert.getUser().getId());
				}else{
					ySeries[index] = 0;
				}
				index ++;
			}
			
			
			
			
			ILineSeries series = (ILineSeries) chart.getSeriesSet().createSeries( SeriesType.LINE, expert.getUser().getUserName());
			series.setYSeries(ySeries);
			series.setSymbolSize(5);
			series.setLineStyle(LineStyle.NONE);
			
			if(expertIndex >= colorList.size() ){
				int result =(expertIndex - colorList.size()) % 3;
				if(result == 0){
					if(firstValue[0] - size > 0 && firstValue[1] + size < 255 && firstValue[2] + size < 255){
						firstValue[0] = firstValue[0] - size;
						firstValue[1] = firstValue[1] + size;
						firstValue[2] = firstValue[2] + size;
					}
					series.setLineColor(ResourceManager.getColor(firstValue[0], firstValue[1], firstValue[2]));//设置线颜色
					series.setSymbolColor(ResourceManager.getColor(firstValue[0], firstValue[1], firstValue[2]));//设置值点颜色
				}else if(result == 1){
					if(secondValue[0] + size < 255 && secondValue[1] - size > 0 && secondValue[2] + size < 255 ){
						secondValue[0] = secondValue[0] + size;
						secondValue[1] = secondValue[1] - size;
						secondValue[2] = secondValue[2] + size;
					}
					series.setLineColor(ResourceManager.getColor(secondValue[0], secondValue[1], secondValue[2]));//设置线颜色
					series.setSymbolColor(ResourceManager.getColor(secondValue[0], secondValue[1], secondValue[2]));//设置值点颜色
				}else if(secondValue[0] + size < 255 && secondValue[2] - size > 0 && secondValue[1] + size < 255 ){
					if(thirdValue[1] - size > 0 && thirdValue[2] + size < 255){
						thirdValue[2] = thirdValue[2] - size;
						thirdValue[1] = thirdValue[1] + size;
						thirdValue[0] = thirdValue[0] + size;
					}
					series.setLineColor(ResourceManager.getColor(thirdValue[0], thirdValue[1], thirdValue[2]));//设置线颜色
					series.setSymbolColor(ResourceManager.getColor(thirdValue[0], thirdValue[1], thirdValue[2]));//设置值点颜色
				}
				
			}else{
				series.setLineColor(colorList.get(expertIndex));//设置线颜色
				series.setSymbolColor(colorList.get(expertIndex));//设置值点颜色
			}
			
			series.setLineWidth(5);
			expertIndex ++;

			
		}
		for(Expert expert : expertList){
			int index = 0;
			for(EvaluationFactor factor : factorList){
				if(null != evaluationFactorResultMap.get(suplier.getId() + factor.getId() + expert.getUser().getId())){
					avgSeries[index] = avgSeries[index] + evaluationFactorResultMap.get(suplier.getId() + factor.getId() + expert.getUser().getId());
				}
				index ++;
			}
		}
		for(int i = 0; i < avgSeries.length; i++ ){
			avgSeries[i] = avgSeries[i] / expertList.size();
		}
		ILineSeries avgseries = (ILineSeries) chart.getSeriesSet().createSeries( SeriesType.LINE, "平均线");
		avgseries.setYSeries(avgSeries);
		avgseries.enableArea(true);
		avgseries.setLineWidth(2);
		avgseries.setLineColor(avgScoreLineColor);
		avgseries.setSymbolColor(avgScoreLineColor);
		avgseries.setSymbolType(PlotSymbolType.NONE);
		
		double[] fullMarksSeries = new double[factorList.size()];
		for(int i = 0; i < factorList.size(); i++ ){
			if(HightScore.compareTo(factorList.get(i).getFactorScore()) < 0){
				HightScore = factorList.get(i).getFactorScore();
			}
			fullMarksSeries[i] = factorList.get(i).getFactorScore().doubleValue();
		}
		ILineSeries fullseries = (ILineSeries) chart.getSeriesSet().createSeries( SeriesType.LINE, "满分线");
		fullseries.setLineWidth(2);

		fullseries.setYSeries(fullMarksSeries);
		fullseries.setSymbolType(PlotSymbolType.NONE);
		fullseries.enableArea(true);
//		
		fullseries.setLineColor(fullScoreLineColor);
		fullseries.setSymbolColor(fullScoreLineColor);
		
		
		double[] hignseriesValue = new double[factorList.size()]; 
		HightScore = BigDecimalUtils.add(HightScore, new BigDecimal(2));
		for(int i = 0 ; i < hignseriesValue.length; i++){
			hignseriesValue[i] = HightScore.doubleValue();
		}
		ILineSeries hignseries = (ILineSeries) chart.getSeriesSet().createSeries( SeriesType.LINE, ".");
		hignseries.setDescription("");
		hignseries.setLineStyle(LineStyle.NONE);
		hignseries.setYSeries(hignseriesValue);
		hignseries.setSymbolType(PlotSymbolType.NONE);
//		hignseries.enableArea(true);
//		
		//零分线
		double[] zeroLine = new double[factorList.size()]; 
		ILineSeries zeroLineSeries = (ILineSeries) chart.getSeriesSet().createSeries( SeriesType.LINE, "..");
		zeroLineSeries.setDescription("");
		zeroLineSeries.setLineStyle(LineStyle.NONE);
		zeroLineSeries.setYSeries(zeroLine);
		zeroLineSeries.setSymbolType(PlotSymbolType.NONE);
		
		chart.getAxisSet().adjustRange();
		composite.layout();
		scrolledComposite.setContent(composite);
		scrolledComposite.setMinSize(composite.computeSize(SWT.DEFAULT, SWT.DEFAULT));
		chart.getAxisSet().adjustRange();
		composite.layout();
	}
	
	
	/**
	 * @Description: 装载具体评审点
	 * @author guoyr 
	 */
	private void setFactorList(){
		evaluationFactorList = new ArrayList<EvaluationFactor>(); 
		selectFactorList = new ArrayList<EvaluationFactor>(); 
		List<EvaluationFactor> evaluationTopFactorList = evaluationFactorService.getEvaluationStepFactorTreeList(tenderProjectEntity.getId(), evaluationStep);
		if(null != evaluationTopFactorList && evaluationTopFactorList.size() > 0){
			for(EvaluationFactor topFactor : evaluationTopFactorList){
				initAddChildEvaluationFactor(topFactor);
			}
		}
	}
	/**
	 * @Description: 获取评审指标列表
	 * @author guoyr
	 * @return 
	 */
	private String[] getFactors(List<EvaluationFactor> selectEvaluationFactorList){
		StringBuffer factorNames = new StringBuffer();
		int index = 1;
		for(EvaluationFactor factor : selectEvaluationFactorList){
			factorNames.append(factor.getFactorName());
			if(index < selectEvaluationFactorList.size()){
				factorNames.append(",");
			}
		}
		return factorNames.toString().split(",");
	}
	
	
	/**
	 * @Description: 设置评标结果数据集
	 * @author guoyr 
	 */
	private void setDataMap(){
		List<EvaluationFactorResult> evaluationFactorResultList = evaluationFactorResultService.getEvaluationFactorResultList(tenderProjectEntity.getId(), evaluationStep.getId());
		for(EvaluationFactorResult evaluationFactorResult : evaluationFactorResultList){
			evaluationFactorResultMap.put(evaluationFactorResult.getSupplierId() + evaluationFactorResult.getFactorId() + evaluationFactorResult.getUserId() ,evaluationFactorResult.getEvaluationFinalScore().doubleValue());
		}
	}
	/**
	 * @Description: 递归获取所有的评审叶子指标
	 * @author guoyr
	 * @param evalFactor
	 */
	private void initAddChildEvaluationFactor(EvaluationFactor evalFactor){
		if(null != evalFactor.getChildFactorList() && evalFactor.getChildFactorList().size() > 0){
			for(EvaluationFactor childFactor : evalFactor.getChildFactorList()){
				childFactor.setLevel(evalFactor.getLevel() + 1);
				initAddChildEvaluationFactor(childFactor);
			}
		}else {
			evaluationFactorList.add(evalFactor);
		}
	}
}
