package com.gx.obe.common.composite;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.text.SimpleDateFormat;

import org.apache.log4j.Logger;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;

import com.gx.obe.common.file.enumeration.FileConstants;
import com.gx.obe.config.ConfigConstants;
import com.gx.obe.config.utils.PropertiesUtils;
import com.gx.obe.message.Messages;
import com.gx.obe.web.entity.Supplier;
import com.gx.obe.web.entity.tender.TenderProjectEntity;
import com.gx.tools.ole.DocumentFactory;
import com.gx.tools.ole.DocumentLoader;
import com.lowagie.text.Cell;
import com.lowagie.text.Document;
import com.lowagie.text.DocumentException;
import com.lowagie.text.Element;
import com.lowagie.text.Font;
import com.lowagie.text.PageSize;
import com.lowagie.text.Paragraph;
import com.lowagie.text.Table;
import com.lowagie.text.pdf.BaseFont;
import com.lowagie.text.pdf.PdfWriter;

/**
 * @Description:签到结果查看面板 
 * @author guoyr
 */
public class SupplierSignatureResultComposite extends Composite {

	public TenderProjectEntity tenderProjectEntity;
	public Supplier supplier;
	private Composite pdfLoadComposite;
	private File signatureFile;
	Logger LOG = Logger.getLogger(SupplierSignatureResultComposite.class);
	private DocumentLoader documentLoader;

	/** 
	 * @author guoyr
	 * @param parent
	 * @param style 
	 */
	public SupplierSignatureResultComposite(Composite parent, int style) {
		super(parent, style);
		GridLayout gridLayout = new GridLayout(1, false);
		gridLayout.marginHeight = 1;
		gridLayout.marginWidth = 1;
		setLayout(gridLayout);
		
		Composite composite = new Composite(this, SWT.NONE);
		composite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
		GridLayout gl_composite = new GridLayout(1, false);
		gl_composite.marginWidth = 0;
		gl_composite.marginHeight = 0;
		gl_composite.verticalSpacing = 0;
		composite.setLayout(gl_composite);
		
		pdfLoadComposite = new Composite(composite, SWT.NONE);
		pdfLoadComposite.setLayout(new FillLayout(SWT.HORIZONTAL));
		pdfLoadComposite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
	}
	
	/**
	 * @Description: 刷新
	 * @author guoyr
	 * @param tenderProjectEntity
	 * @param supplier
	 */
	public void refresh(TenderProjectEntity tenderProjectEntity, Supplier supplier){
		this.tenderProjectEntity = tenderProjectEntity;
		this.supplier = supplier;
		String signPdfPath = FileConstants.getSupplierSignFilePath(tenderProjectEntity.getId(), supplier.getOrgCode());
		writePDF(signPdfPath);
		showPDF(signPdfPath);
	}

	/**
	 * @Description:展示pdf签章界面，加载pdf进行签章操作 
	 * @author guoyr
	 * @param filePath
	 */
	private void showPDF(String filePath){
		for(Control control : pdfLoadComposite.getChildren()){
			control.dispose();
		}
		File documentFile = new File(filePath);
		documentLoader = DocumentFactory.getDocumentLoader(null != documentFile&& documentFile.exists()?documentFile.getAbsolutePath():null, PropertiesUtils.getProperty(ConfigConstants.CA_SIGNATURE_PDF, ""));
		documentLoader.openDocument(pdfLoadComposite, true, false, false, false, false);
		layout();
	}

	/**
	 * @Description: 写入pdf文件
	 * @author guoyr
	 * @param filePath
	 */
	
	private  void writePDF(String filePath) {
		signatureFile = new File(filePath);
		//判断当前本地 文件目录是否存在，不存在就创建目录。
		if(!signatureFile.getParentFile().exists()){
			signatureFile.getParentFile().mkdirs();
		}
		//判断当前 要写入的本地文件 是否存在，不存在就创建新的文件
		if(signatureFile.exists()){
			signatureFile.delete();
		}
		try {
			signatureFile.createNewFile();
		} catch (IOException e) {
			LOG.error("", e);
		}
		//判断当前投标人签到文件是否存在，直接加载 签到文件数据 ，如果 不存在 ，则需要写入签到数据
		if(null != supplier.getSignFile()){
//		if(null != supplier.getSignTime() && null != supplier.getSignFile()){
			FileOutputStream out = null;
			byte[] bytes = supplier.getSignFile();
			try {
				out = new FileOutputStream(signatureFile);
				out.write(bytes);
			} catch (Exception e) {
				LOG.error("", e);
			}finally{
				try {
					out.flush();
					out.close();
				} catch (IOException e) {
					LOG.error("", e);
				}
				
			}
			
		}else{
			//投标人还没签到，所以没有对应 签到文件 
			FileOutputStream out = null;
			try {
				//创建文档,设置页面大小,      左、右、上和下页边距。
				Document document = new Document(PageSize.A4, 50, 50, 50, 50);
				//document是创建的文档,out是输出
				PdfWriter writer =PdfWriter.getInstance(document, new FileOutputStream(signatureFile));
				writer.setViewerPreferences(PdfWriter.PageModeUseOutlines);
				document.open();
				//----------------
				BaseFont bfChinese = BaseFont.createFont("STSong-Light", "UniGB-UCS2-H",BaseFont.NOT_EMBEDDED);   
				
				Font FontChinese = new Font(bfChinese, 22, Font.NORMAL);
				if(null != tenderProjectEntity.getParentProjectEntity()){
					
					Paragraph projectMessage = new Paragraph("["+tenderProjectEntity.getParentProjectEntity().getTenderNo() + "]"+tenderProjectEntity.getParentProjectEntity().getTenderName(),FontChinese);
					projectMessage.setAlignment(Element.ALIGN_CENTER);
					
					FontChinese = new Font(bfChinese, 20, Font.NORMAL);
					Paragraph packageMessage = new Paragraph("["+tenderProjectEntity.getTenderNo() + "]"+tenderProjectEntity.getTenderName(),FontChinese);
					packageMessage.setAlignment(Element.ALIGN_CENTER);
					document.add(projectMessage);
					document.add(packageMessage);
				}else {
					
					Paragraph projectMessage = new Paragraph("["+tenderProjectEntity.getTenderNo() + "]"+tenderProjectEntity.getTenderName(),FontChinese);
					projectMessage.setAlignment(Element.ALIGN_CENTER);
					document.add(projectMessage);
				}
				
				Table table = new Table(2);
				table.setBorderWidth(0f);
				table.setTableFitsPage(true);
				table.setCellsFitPage(true);
				table.setPadding(2);
				table.setSpacing(2);// 间隔
				table.setAlignment(Element.ALIGN_CENTER);
				
				Paragraph p = null;
				Cell cell = null;
				
				// 投标人名称
				FontChinese = new Font(bfChinese, 20, Font.NORMAL);
				p =  new Paragraph(supplier.getSupplierName(),FontChinese);
				cell = new Cell(p);
				cell.setColspan(2);
				cell.setBorderWidth(0f);
				cell.setHorizontalAlignment(Element.ALIGN_CENTER);
				table.addCell(cell);
				
				FontChinese = new Font(bfChinese, 14, Font.NORMAL);
				p =  new Paragraph("联系人：",FontChinese);
				cell = new Cell(p);
				cell.setBorderWidth(0f);
				cell.setHorizontalAlignment(Element.ALIGN_RIGHT);
				table.addCell(cell);
				
				p =  new Paragraph(supplier.getSignPerson(),FontChinese);
				cell= new Cell(p);
				cell.setBorderWidth(0f);
				table.addCell(cell);
				
				p =  new Paragraph(Messages.LinkerTel+"：",FontChinese);
				cell= new Cell(p);
				cell.setBorderWidth(0f);
				cell.setHorizontalAlignment(Element.ALIGN_RIGHT);
				table.addCell(cell);
				
				p =  new Paragraph(supplier.getSignPersonTel(),FontChinese);
				cell= new Cell(p);
				cell.setBorderWidth(0f);
				table.addCell(cell);
				
				if(null != supplier.getEmail() && supplier.getEmail().length() > 0){
					p =  new Paragraph("邮箱地址：",FontChinese);
					cell= new Cell(p);
					cell.setBorderWidth(0f);
					cell.setHorizontalAlignment(Element.ALIGN_RIGHT);
					table.addCell(cell);
					
					p =  new Paragraph(supplier.getEmail(),FontChinese);
					cell= new Cell(p);
					cell.setBorderWidth(0f);
					table.addCell(cell);
				}
				
				p =  new Paragraph(Messages.CheckInTime+"：",FontChinese);
				cell= new Cell(p);
				cell.setBorderWidth(0f);
				cell.setHorizontalAlignment(Element.ALIGN_RIGHT);
				table.addCell(cell);
				p =  new Paragraph(null != supplier.getSignTime() ? new SimpleDateFormat("yyyy-MM-dd : HH : mm : ss").format(supplier.getSignTime()) : "" ,FontChinese);
				cell= new Cell(p);
				cell.setBorderWidth(0f);
				table.addCell(cell);
				
//				p =  new Paragraph("投递标书方式：",FontChinese);
//				cell= new Cell(p);
//				cell.setBorderWidth(0f);
//				cell.setHorizontalAlignment(Element.ALIGN_RIGHT);
//				table.addCell(cell);
//				p =  new Paragraph(SupplierEnum.getSupplierTenderType(supplier.getBidFileType()),FontChinese);
//				cell= new Cell(p);
//				cell.setBorderWidth(0f);
//				table.addCell(cell);
				
				document.add(table);
//				
//				FontChinese = new Font(bfChinese, 22, Font.NORMAL);
//				Paragraph signState = new Paragraph("(线下签到！)",FontChinese);
//				signState.setAlignment(Element.ALIGN_CENTER);
//				document.add(signState);
				
				document.close();
			} catch (DocumentException e) {
				LOG.error("", e);
			} catch (IOException e) {
				LOG.error("", e);
			} finally {
				try {
					if (out != null){
						out.close();
					}
				} catch (IOException e1) {
				}
			}
		}
	}
	
	@Override
	protected void checkSubclass() {
		// Disable the check that prevents subclassing of SWT components
	}
}
